@@ -23,7 +23,7 @@ our @ISA = qw (Exporter);
 our @EXPORT = qw ($LS $CAT $CP $SU $RM $PS $GREP $ECHO $IFCONFIG $PING $READLINK
                   $MV $LN $UNAME $WHOAMI $WHICH $STTY $SERVICE $CRM $CRM_MON
                   $CRM_ATTRIBUTE $CRM_RESOURCE $SYNC $TAIL $PMSTART $PMSTOP
-                  $CRM_NODE $ROUTE $PIDOF $GZIP $BZIP2);
+                  $CRM_NODE $ROUTE $PIDOF $GZIP $BZIP2 $LSOF $KILL);
 
 our $LS            = "/bin/ls";
 our $CAT           = "/bin/cat";
@@ -56,5 +56,7 @@ our $ROUTE         = "/sbin/route";
 our $PIDOF         = "/sbin/pidof";
 our $GZIP          = "/bin/gzip";
 our $BZIP2         = "/usr/bin/bzip2";
+our $LSOF          = "/usr/sbin/lsof";
+our $KILL          = "/usr/bin/kill";
 
 1;
