@@ -374,6 +374,12 @@ _SWITCHOVER_USAGE_
     COMMON_MS0043       => "ifcheckd has stopped on another node.\n",
     COMMON_MS0044       => "Could not check the status of archive directory: [0]\n",
     COMMON_MS0045       => "The format of fist line of the backup label file is invalid: [0]\n",
+    COMMON_MS0046       => "DB cluster status check failed: [0]\n",
+    COMMON_MS0047       => "There are processes accessing the DB cluster.\n[0]",
+    COMMON_MS0048       => "Failed to create lock file([0]). : [1]\n",
+    COMMON_MS0049       => "[0] has already started on this node.\n",
+    COMMON_MS0050       => "Failed to create lock file([0]).\n",
+    COMMON_MS0051       => "Failed to delete lock file([0]). : [1]\n",
 };
 
 1;
