@@ -8,7 +8,7 @@
 # 特記事項:
 # なし
 #
-# Copyright (c) 2012-2015, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+# Copyright (c) 2012-2017, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
 #
 #####################################################################
 package PGRex;
@@ -18,6 +18,8 @@ use strict;
 use Net::OpenSSH;
 use PGRex::command;
 use Class::Struct;
+use Fcntl;
+use Errno;
 
 BEGIN {
     if ($ENV{'LANG'} =~ m/ja/i){
@@ -44,7 +46,7 @@ struct Pg_dir_state => {
 };
 
 use constant {
-    VERSIONNUM       => "1.7.1",
+    VERSIONNUM       => "1.8.1",
     VERSIONINFO      => "[0] (pg-rex_operation_tools) [1]\n",
     CONFIG_PATH      => "/etc/",
     CONFIG_FILENAME  => "pg-rex_tools.conf",
@@ -54,9 +56,12 @@ use constant {
     HACF_FILENAME    => "corosync.conf",
     AUTHKEY_FILENAME => "authkey",
     RA_TMPDIR        => "/var/lib/pgsql/tmp",
-    LOCK_FILENAME    => "PGSQL.lock"
+    LOCK_FILENAME    => "PGSQL.lock",
+	PID_FILENAME     => "pg-rex_tools.pid"
 };
 
+our $additional_information_mode = 0;
+
 sub pacemaker_running{
     my ($ssh_info) = @_;
     my $result;
@@ -923,35 +928,14 @@ sub compare_lsn{
 sub get_controldata_value{
     my (@controldata_strings) = @_;
     my %controldata_value;
-    my @check_key_list = ("update_time", "database", "time_line_id", "xlog_location");
+    my @check_key_list = ("pg_control last modified", "Database system identifier", "Latest checkpoint's TimeLineID", "Latest checkpoint's REDO location", "Latest checkpoint's REDO WAL file");
 
-    foreach my $line (@controldata_strings){
-        # pg_control last modified format : pg_control last modified: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
-        if ($line =~ /^pg_control\s+last\s+modified:\s+(.+)$/){
-            $controldata_value{'update_time'} = $1;
-        }
-        # Database system identifier format : Database system identifier: <Database>
-        if ($line =~ /^Database\s+system\s+identifier:\s+([0-9A-F]+)$/){
-            $controldata_value{'database'} = $1;
-        }
-        # TimeLineID format : Latest checkpoint's TimeLineID: <TimeLineID>
-        # <TimeLineID>は10進数
-        if ($line =~ /^Latest\s+checkpoint's\s+TimeLineID:\s+([0-9A-F]+)$/){
-            $controldata_value{'time_line_id'} = $1;
-        }
-        # location format : Latest checkpoint's REDO location: <location>
-        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+location:\s+([0-9A-F\/]+)$/){
-            $controldata_value{'xlog_location'} = $1;
-        }
-        # checkpoint time format : Time of latest checkpoint: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
-        if ($line =~ /^Time\s+of\s+latest\s+checkpoint:\s+(.+)$/){
-            $controldata_value{'checkpoint_time'} = $1;
-        }
-        # WAL file name format : Latest checkpoint's REDO WAL file: <WAL>
-        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+WAL\s+file:\s+([0-9A-F]+)$/){
-            $controldata_value{'wal_filename'} = $1;
-        }
-    }
+	my @key_val;
+
+	foreach my $line (@controldata_strings) {
+		@key_val = split(/:/, $line);
+		($controldata_value{$key_val[0]} = $key_val[1]) =~ s/^ *(.*?) *$/$1/;
+	}
 
     foreach my $key (@check_key_list) {
         if (!exists($controldata_value{$key})){
@@ -960,7 +944,7 @@ sub get_controldata_value{
     }
 
 
-    return %controldata_value;
+    return \%controldata_value;
 }
 
 sub get_start_wal_filename {
@@ -977,9 +961,9 @@ sub get_recoverywal {
     my (@controldata_strings) = @_;
     my $wal = '';
 
-    my %controldata_value = get_controldata_value(@controldata_strings);
+    my $controldata_value = (get_controldata_value(@controldata_strings));
 
-    $wal = get_xlog_filename($controldata_value{'xlog_location'}, $controldata_value{'time_line_id'});
+    $wal = get_xlog_filename($controldata_value->{"Latest checkpoint's REDO location"}, $controldata_value->{"Latest checkpoint's TimeLineID"});
 
     return $wal;
 }
@@ -1087,7 +1071,7 @@ sub check_support_version {
     # PostgreSQL がサポート対象バージョンであるかを確認する
     @version_factor = split(/\./, get_pg_version($postgres_path, $exec_user));
     if ($version_factor[0] != 9 ||
-        ($version_factor[1] != 4 && $version_factor[1] != 5)){
+        ($version_factor[1] != 5 && $version_factor[1] != 6)){
             printlog("ERROR", COMMON_MS0037, get_pg_version($postgres_path), $exec_user);
     }
 }
@@ -1113,7 +1097,7 @@ sub ifcheckd_running {
 sub get_pg_command_path {
     my ($pg_path, $exec_user) = @_;
     my %command_path;
-    my @command_list = ("postgres", "psql", "pg_controldata", "pg_basebackup", "vacuumdb", "pg_xlogdump");
+    my @command_list = ("postgres", "psql", "pg_controldata", "pg_basebackup", "vacuumdb", "pg_xlogdump", "pg_rewind", "pg_ctl");
     my $pg_command_user = "postgres";
 
     if (!$pg_path){
@@ -1208,8 +1192,14 @@ sub printlog {
     }
 
     if ($log_level eq "ERROR"){
-        die $message;
+        print $message;
+		exit(1);
     }
+	elsif ($log_level eq "DEBUG"){
+		if ($PGRex::common::additional_information_mode) {
+			print $message;
+		}
+	}
     else{
         print $message;
     }
@@ -1264,4 +1254,96 @@ sub get_ssh_passwd {
     return $passwd;
 }
 
+
+sub check_dbcluster_access {
+    my ($cluster_path) = @_;
+    my %procs;
+    my $result = "";
+
+	stat $LSOF || printlog("ERROR", COMMON_MS0046, $!);
+
+    open my $in, '-|', "$LSOF +D $cluster_path -F cuLR0 2> /dev/null" || printlog("ERROR", COMMON_MS0046, $!);
+	# Suppress display of stderr only, because error of lsof command is unknown
+
+    while (<$in>) {
+        my $pid;
+
+        while (/([a-zA-Z])([^\x00]+)\x00/g) {
+            if ($1 eq "p") {
+                $pid = $2;
+            } else {
+                $procs{$pid}{$1} = $2;
+            }
+        }
+    }
+    close $in;
+
+    if (keys %procs) {
+        foreach my $key (keys %procs) {
+            # filter out forked children
+            next if (defined $procs{$procs{$key}{R}});
+
+            $result .= "PID: $key / COMMAND: $procs{$key}{c}, USER=$procs{$key}{L}($procs{$key}{u})\n";
+        }
+        printlog("ERROR", COMMON_MS0047, $result);
+    }
+}
+
+
+sub create_pid_file {
+    my $pid_fullname = RA_TMPDIR."/".PID_FILENAME;
+    my $pid;
+    my $exit_code;
+
+    for(my $ntries = 0; ; $ntries++) {
+
+        # create pid file
+        if (sysopen my $handle, "$pid_fullname", O_WRONLY|O_CREAT|O_EXCL) {
+            print $handle "$$";
+            close $handle;
+            return;
+        } elsif ($! != Errno::EEXIST) {
+            printlog("ERROR", COMMON_MS0048, PID_FILENAME, $!);
+        }
+
+		if ($ntries > 5) {
+			last;
+		}
+		$ntries != 0 && sleep 1;
+
+        # check pid file
+		open FILE, "< $pid_fullname" || continue;
+		$pid = <FILE>; # If it can not read, $pid is UNDEF
+		close FILE;
+
+		# other pg-rex_tool has already started
+		if (defined($pid)) {
+			chomp $pid;
+			if ($pid ne "") {
+				`$KILL -s 0 $pid 2> /dev/null`;
+				$exit_code = $? >> 8;
+				if ($exit_code == 0) {
+					printlog("ERROR", COMMON_MS0049, $pid);
+				}
+			}
+		}
+
+        # unlink pid file
+        unlink $pid_fullname;
+		if ($! == Errno::EISDIR) {
+			printlog("ERROR", COMMON_MS0051, $!);
+		}
+    }
+
+    printlog("ERROR", COMMON_MS0050, PID_FILENAME);
+}
+
+sub unlink_pid_file {
+    my $pid_fullname = RA_TMPDIR."/".PID_FILENAME;
+
+    if (!unlink $pid_fullname) {
+        printlog("LOG", COMMON_MS0051, PID_FILENAME, $!);
+    }
+}
+
 1;
